package com.huimv.process.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.huimv.process.dao.entity.ImageCatalogEntity;
import com.huimv.process.dao.entity.ImagePathEntity;
import com.huimv.process.dao.repo.ImageCatalogRepo;
import com.huimv.process.dao.repo.ImagePathEntityRepository;
import com.huimv.process.service.IImageService;
import com.huimv.process.utils.DateUtil;
import com.huimv.process.utils.PictureUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;
import sun.misc.BASE64Decoder;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.sql.Timestamp;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
@Slf4j
public class ImageServiceImpl implements IImageService {
    @Autowired
    private PictureUtil pictureUtil;
    @Autowired
    private ImagePathEntityRepository imagePathRepo;
    @Autowired
    private RestTemplate restTemplate;
    @Autowired
    private com.huimv.process.utils.utils utils;
    @Autowired
    private ImageCatalogRepo imageCatalogRepo;
    @Autowired
    private DateUtil dateUtil;

    /**
     * @Method : saveImage
     * @Description :
     * @Params : [imageBase64, name]
     * @Return : com.alibaba.fastjson.JSONObject
     * @Author : ZhuoNing
     * @Date : 2021/1/21
     * @Time : 17:31
     */
    @Override
    public JSONObject saveImage(String imageBase64, String cameraName) {
        String uploadPath = "uploadImages";
        //
        String os = System.getProperty("os.name");
        String userDir = System.getProperty("user.dir");
        String createPath = "";
        if (os.toLowerCase().indexOf("windows") != -1) {
            // Windows
            uploadPath = "\\" + uploadPath + "\\"+cameraName+"\\";
            createPath = userDir + uploadPath;
        } else {
            // Linux
            uploadPath = "/" + uploadPath + "/"+cameraName+"/";
            createPath = userDir + uploadPath;
        }
        // 时间戳
        long timeMillis = System.currentTimeMillis();
        //判断文件目录是否存在
        File dir = new File(createPath);
        if (!dir.exists()) { // && dir.isDirectory()
            dir.mkdirs();
            // 目录入库
            ImageCatalogEntity imageCatalogEntity = new ImageCatalogEntity();
            imageCatalogEntity.setCatalogName(cameraName);
            imageCatalogEntity.setCatalogPath(cameraName);
            imageCatalogRepo.saveAndFlush(imageCatalogEntity);
        }


        // type1
//        String imgPath = uploadPath + name + "_"+ timeMillis + ".jpg";
        // type2
//        StringBuilder imgBuild = new StringBuilder();
//        String imgPath = imgBuild.append(uploadPath).append(name).append("_").append(timeMillis).append(".jpg").toString();
        // type 3
        StringBuilder fileNameBuild = new StringBuilder();
//        String fileName = fileNameBuild.append(cameraName).append("_").append(timeMillis).append(".jpg").toString();
        String fileName = fileNameBuild.append(cameraName).append("_").append(dateUtil.formatLongTimeToImageName(timeMillis)).append(".jpg").toString();
        String relativePath = uploadPath + fileName;
        String imgPath = userDir + relativePath;
        JSONObject resultJo = new JSONObject();
        resultJo.put("result", pictureUtil.base64ToImage(imageBase64, imgPath));
        resultJo.put("imgPath", imgPath);
        resultJo.put("relativePath", uploadPath + fileName);
        resultJo.put("fileName", fileName);
        log.info("resultJo>>" + resultJo);
        // 保存
        return resultJo;
    }

    /**
     * @Method : addImage
     * @Description :
     * @Params : [name, imgPath, relativePath, fileName]
     * @Return : boolean
     * @Author : ZhuoNing
     * @Date : 2021/1/21
     * @Time : 17:31
     */
    @Override
    public boolean addImage(String name, String imgPath, String relativePath, String fileName, String aiResult, int aiTimeUsed, String aiData, Timestamp createDate,int pigQuantity) {
        ImagePathEntity imagePathEntity = new ImagePathEntity();
        imagePathEntity.setName(name);
        imagePathEntity.setImgPath(imgPath);
        imagePathEntity.setRelativePath(relativePath);
        imagePathEntity.setFileName(fileName);
        imagePathEntity.setEffective(1);
        imagePathEntity.setAiResult(aiResult);
        imagePathEntity.setAiTimeUsed(aiTimeUsed);
        imagePathEntity.setAiData(aiData);
        imagePathEntity.setCreateDate(createDate);
        imagePathEntity.setPigQuantity(pigQuantity);
        imagePathRepo.saveAndFlush(imagePathEntity);
        return true;
    }

    /**
     * @Method : getAIResult
     * @Description : 用RestTemplate调取第三方接口进行AI识别出猪
     * @Params : [imgPath]
     * @Return : java.lang.String
     * @Author : ZhuoNing
     * @Date : 2021/1/21
     * @Time : 16:25
     */
    @Override
    public String getAiResult2(String imgPath) throws IOException {
        System.out.println("AI识别图片路径imgPath>>" + imgPath);
        //
        MultiValueMap<String, Object> paramMap = new LinkedMultiValueMap<>();
//            File file=new File("D:\\pic\\微信图片_20210120175259.jpg");
        File file = new File(imgPath);
        byte[] bytesArray = new byte[(int) file.length()];
        FileInputStream fis = new FileInputStream(file);
        fis.read(bytesArray); //read file into bytes[]
        fis.close();
        ByteArrayResource contentsAsResource = new ByteArrayResource(bytesArray) {
            @Override
            public String getFilename() {
                return "imageData";
            }
        };
        paramMap.add("imageData", contentsAsResource);
        System.out.println("AI参数>>" + paramMap);
        JSONObject json = restTemplate.postForObject("http://122.112.224.199:31288/api/piggy", paramMap, JSONObject.class);
        System.out.println("post json : " + json);
        return null;
    }

    /**
     * @Method :
     * @Description :
     * @Params :
     * @Return :
     * @Author : ZhuoNing
     * @Date : 2021/1/21
     * @Time : 17:44
     */
    @Override
    public JSONObject getAiResult(String imgPath,String fileName) throws IOException {
        log.info("Ai识别图片路径>>" + imgPath);
        //      FileSystemResource fileResource = new FileSystemResource(new File("d://image"));
        BASE64Decoder decoder = new BASE64Decoder();
//      byte[] rawBytes = utils.File2byte("D:\\pic\\20210120175259.jpg");//将图片转为byte[]
        byte[] rawBytes = utils.File2byte(imgPath);//将图片转为byte[]
//        byte[] rawBytes = decoder.decodeBuffer(imgPath);
//        String suffix = utils.getSuffix(rawBytes);//获取图片的后缀名，也可以是其他任意文件名
//        System.out.println("suffix>>"+suffix);
//        String fileName = "myImage" + suffix;//
        ByteArrayResource fileResource = new ByteArrayResource(rawBytes) {
            @Override
            public String getFilename() {
                return fileName;
            }
        };
        MultiValueMap<String, Object> postParameters = new LinkedMultiValueMap<String, Object>();
        postParameters.add("imageData", fileResource);
//        HttpHeaders headers = new HttpHeaders();
//        headers.setContentType(MediaType.MULTIPART_FORM_DATA);
//        HttpEntity<MultiValueMap<String, Object>> requestEntity  = new HttpEntity<MultiValueMap<String, Object>>(postParameters, headers);
        RestTemplate restTemplate = new RestTemplate();
//        String url=localUri+"/image/upload";
//        BasicResponse basicResponse=restTemplate.postForObject(url,postParameters,BasicResponse.class);
        String url = "http://122.112.224.199:31288/api/piggy";
        return restTemplate.postForObject(url, postParameters, JSONObject.class);
    }

//    @Override
//    public boolean saveImage(String imageBase64, String name) {
//        String uploadPath = "";
//        //
//        String os = System.getProperty("os.name");
//        String userDir = System.getProperty("user.dir");
//        if (os.toLowerCase().indexOf("windows") != -1) {
//            // Windows
//            uploadPath = userDir + "\\uploadImages\\";
//        } else {
//            // Linux
//            uploadPath = userDir+ "/uploadImages/";
//        }
//        // 时间戳
//        long timeMillis = System.currentTimeMillis();
//        //判断文件目录是否存在
//        File dir = new File(uploadPath);
//        if (!dir.exists()) { // && dir.isDirectory()
//            dir.mkdirs();
//        }
////        String imgPath = uploadPath + name + "_"+ timeMillis + ".jpg";
//        StringBuilder imgBuild = new StringBuilder();
//        String imgPath = imgBuild.append(uploadPath).append(name).append("_").append(timeMillis).append(".jpg").toString();
//        // 保存
//        return pictureUtil.base64ToImage(imageBase64, imgPath);
//    }
}
